#ifndef INC_LISTE_HXX
#define INC_LISTE_HXX

#ifndef INC_TYPES_HXX
#include "types.hxx"
#endif

#include <stdarg.h>
#include <string.h>



template <class T> class Liste 
{
	public:
		Liste();
		~Liste();
		void Append(T* Data);
		void Delete();
		int Find(T* d);
		T* First();
		T* Next();
		T* Last();
		T* Previous();
		T* GetCurrentData();
		void Insert(T* Data);
		long ForEach(long (*EnumFunction) (T*, va_list) , ...);
		int GetCount();
		Liste<T>& operator =(Liste<T>& l);
		Liste<T>& operator +=(Liste<T>& l);
		Liste(Liste<T>& l);
		void DeleteAll();
	protected:
		int Count;
	private:
		Element<T>* pFirst;
		Element<T>* pCurrent;
		Element<T>* pLast;
};

template <class T> Liste<T> :: Liste()
{
	pFirst=pLast=pCurrent=0;
	Count=0;
}

template <class T> Liste<T> :: ~Liste()
{
	DeleteAll();
}

template <class T> void Liste<T> :: Append(T* Data)
{
	Element<T>* e=new Element<T>(Data, pLast);
	if (e) {
		if (pLast)
			pLast->SetNext(e);
		pLast=e;
		if (!pFirst)
			pFirst=e;
		Count++;
	}
	pCurrent=e;
}

template <class T> void Liste<T> :: Delete()
{
	if (pCurrent) {
		Element<T>* p=pCurrent->GetPrevious();
		Element<T>* n=pCurrent->GetNext();
	
		if (p)
			p->SetNext(n);
		else
			pFirst=n;
	
		if (n)
			n->SetPrevious(p);
		else
			pLast=p;
	
		delete pCurrent;
		pCurrent=n;
		Count--;
	}
}

template <class T> int Liste<T> :: Find(T* d)
{
	First();
	for (First(); pCurrent; Next()) {
		if (pCurrent->GetData()==d)
			return 1;
	}
	return 0;
		
}

template <class T> T* Liste<T> :: First()
{
	pCurrent=pFirst;
	if (pCurrent)
		return pCurrent->GetData();
	else
		return 0;

}

template <class T> T* Liste<T> :: Next()
{
	if (pCurrent)
		pCurrent=pCurrent->GetNext();
	if (pCurrent)
		return pCurrent->GetData();
	else
		return 0;

}

template <class T> T* Liste<T> :: Last()
{
	pCurrent=pLast;
	if (pCurrent)
		return pCurrent->GetData();
	else
		return 0;

}

template <class T> T* Liste<T> :: Previous()
{
	if (pCurrent)
		pCurrent=pCurrent->GetPrevious();
	if (pCurrent)
		return pCurrent->GetData();
	else
		return 0;

}

template <class T> T* Liste<T> :: GetCurrentData()
{
	if (pCurrent)
		return pCurrent->GetData();
	else
		return 0;
}

template <class T> void Liste<T> :: Insert(T* Data)
{
	if (!pCurrent)
		Append(Data);
	else {
		Element<T>* e=new Element<T> (Data, pCurrent->GetPrevious(), pCurrent);
		if (e) {
			Element<T>* p=pCurrent->GetPrevious();
			if (p) 
				p->SetNext(e);
			else
				pFirst=e;
			pCurrent->SetPrevious(e);
			Count++;
		}
	}
}

template <class T> long Liste<T> :: ForEach(long (*EnumFunction) (T*, va_list) , ...)
{
	va_list argptr;
	va_start(argptr, EnumFunction);
	First();
	long RetValue=0;
	while (pCurrent) {
		Element<T>* pNext=pCurrent->GetNext();
		T* Data=pCurrent->GetData();
		if (Data) {
			RetValue=(* EnumFunction)(Data, argptr);
		};	
		if (RetValue) return RetValue;
		pCurrent=pNext;
	};
	va_end(argptr);
	return RetValue;

}

template <class T> int Liste<T> :: GetCount()
{
	return Count;
}

template <class T> Liste<T>& Liste<T> :: operator =(Liste<T>& l)
{
	DeleteAll();
	Element<T>* pOldCurrent=l.pCurrent;
	T* t;
	for (t=l.First(); t; t=l.Next())
		Append(t, FALSE);
	l.pCurrent=pOldCurrent;
	
	return *this;
}

template <class T> Liste<T>& Liste<T> :: operator +=(Liste<T>& l)
{
	Element<T>* pOldCurrent=l.pCurrent;
	T* t;
	for (t=l.First(); t; t=l.Next())
		Append(t, FALSE);
	l.pCurrent=pOldCurrent;
	return *this;
}

template <class T> Liste<T> :: Liste(Liste<T>& l)
{
	pCurrent=pFirst=pLast=0;
	Count=0;
	
	Element<T>* pOldCurrent=l.pCurrent;
	T* t;
	for (t=l.First(); t; t=l.Next())
		Append(t, FALSE);
	l.pCurrent=pOldCurrent;
}

template <class T> void Liste<T> :: DeleteAll()
{
	while (Last()) {
		Delete();
	}
}


#endif

